unit GetInfo;
interface
uses PLMan, Classes, fmod, fmodtypes;

type TGetSongInfoMode=(gsiFast,gsiThorough,gsiKeepHandle);

function GetSongInfo(var Item:TPlaylistItem; const Ext:string;
                     Dest:TStrings; Mode:TGetSongInfoMode):PFSoundStream;

implementation
uses MP3Tags,Globals,SysUtils;

const MaxGenres=147;
const GenreList:array[0..MaxGenres]of string[23]=(
  'Blues','Classic Rock','Country','Dance','Disco','Funk','Grunge','Hip-Hop',
  'Jazz','Metal','New Age','Oldies','Other','Pop','R&B','Rap','Reggae','Rock',
  'Techno','Industrial','Alternative','Ska','Death Metal','Pranks','Soundtrack',
  'Euro-Techno','Ambient','Trip-Hop','Vocal','Jazz+Funk','Fusion','Trance',
  'Classical','Instrumental','Acid','House','Game','Sound Clip','Gospel',
  'Noise','Alt. Rock','Bass','Soul','Punk','Space','Meditative',
  'Instrumental Pop','Instrumental Rock','Ethnic','Gothic','Darkwave',
  'Techno-Industrial','Electronic','Pop-Folk','Eurodance','Dream',
  'Southern Rock','Comedy','Cult','Gangsta Rap','Top 40','Christian Rap',
  'Pop/Funk','Jungle','Native American','Cabaret','New Wave','Psychedelic',
  'Rave','Showtunes','Trailer','Lo-Fi','Tribal','Acid Punk','Acid Jazz','Polka',
  'Retro','Musical','Rock & Roll','Hard Rock','Folk','Folk/Rock',
  'National Folk','Swing','Fast-Fusion','Bebob','Latin','Revival','Celtic',
  'Bluegrass','Avantgarde','Gothic Rock','Progressive Rock','Psychedelic Rock',
  'Symphonic Rock','Slow Rock','Big Band','Chorus','Easy Listening','Acoustic',
  'Humour','Speech','Chanson','Opera','Chamber Music','Sonata','Symphony',
  'Booty Bass','Primus','Porn Groove','Satire','Slow Jam','Club','Tango',
  'Samba','Folklore','Ballad','Power Ballad','Rhythmic Soul','Freestyle',
  'Duet','Punk Rock','Drum Solo','A Cappella','Euro-House','Dance Hall','Goa',
  'Drum & Bass','Club-House','Hardcore','Terror','Indie','BritPop','Negerpunk',
  'Polsk Punk','Beat','Christian Gangsta Rap','Heavy Metal','Black Metal',
  'Crossover','Contemporary Christian','Christian Rock','Merengue','Salsa',
  'Thrash Metal','Anime','JPop','Synthpop');

function TranslateGenre(const Genre:string):string;
var i,r:integer;
begin
  Result:=Genre;
  val(Genre,i,r);
  if (r=0) AND (i>=0) then
    if i<=MaxGenres then Result:=GenreList[i]
    else if i=255 then Result:='(nicht angegeben)';
end;

function ApplyTemplate(const Artist,Title,Album,Track,Year,Genre,Comment:string):string;
begin
  Result:=TitleTemplate;
  Result:=StringReplace(Result,'$ARTIST',Trim(Artist),[rfReplaceAll,rfIgnoreCase]);
  Result:=StringReplace(Result,'$TITLE',Trim(Title),[rfReplaceAll,rfIgnoreCase]);
  Result:=StringReplace(Result,'$ALBUM',Trim(Album),[rfReplaceAll,rfIgnoreCase]);
  Result:=StringReplace(Result,'$TRACK',Trim(Track),[rfReplaceAll,rfIgnoreCase]);
  Result:=StringReplace(Result,'$YEAR',Trim(Year),[rfReplaceAll,rfIgnoreCase]);
  Result:=StringReplace(Result,'$GENRE',TranslateGenre(Genre),[rfReplaceAll,rfIgnoreCase]);
  Result:=StringReplace(Result,'$COMMENT',Trim(Comment),[rfReplaceAll,rfIgnoreCase]);
  Result:=Trim(Result);
end;

function GetSongInfo(var Item:TPlaylistItem; const Ext:string;
                     Dest:TStrings; Mode:TGetSongInfoMode):PFSoundStream;
var header:mpeghdr; tag:id3info;
    i,max:integer;
    TagType:TFSoundTagFieldType; TagName:PChar; TagValue:PChar; TagLength:integer;
    Artist,Title,Album,Track,Year,Genre,Comment:string;
    PrevTagType:integer; TransTagName:string;
  procedure settag(var dest:string); begin if (length(dest)=0) OR
  (TagType<>FSOUND_TAGFIELD_ID3V2) then dest:=TagValue; end;
begin
  Item.Title:=ExtractFileName(Item.FileName);
  if Assigned(Dest) then begin
    Dest.Clear;
    Dest.Add(ExtractFilePath(Item.FileName));
    Dest.Add(Item.Title);
    Dest.Add('Gre: '+ThousandDots(Item.FileSize));
  end;
  Item.Title:=copy(Item.Title,1,length(Item.Title)-length(Ext));
  if (Mode=gsiFast) AND ((Ext='.mp3') OR (Ext='.mp2')) then begin
    ReadMP3Info(Item.FileName,header,tag);
    if header.valid then begin
      Item.Length:=header.songlen;
      AdjustStrings(Item);
    end;
    if tag.valid then with tag do
      Item.Title:=ApplyTemplate(artist,title,album,'',year,IntToStr(genre),comment);
    Result:=nil; exit;
  end;
  Result:=FSOUND_Stream_Open(PChar(Item.FileName),FSOUND_MPEGACCURATE,0,0);
  if not Assigned(Result) then exit;
  Item.Length:=FSOUND_Stream_GetLengthMs(Result);
  AdjustStrings(Item);
  if Assigned(Dest) then
    Dest.Add('Spieldauer: '+TimeToString(Item.Length)+'.'+
             char(48+Item.Length DIV 100 MOD 10)+
             char(48+Item.Length DIV 10 MOD 10)+
             char(48+Item.Length MOD 10));
  Artist:=''; Title:=''; Album:=''; Track:=''; Year:=''; Genre:=''; Comment:='';
  PrevTagType:=-1;
  if FSOUND_Stream_GetNumTagFields(Result,max) then dec(max) else max:=-1;
  for i:=0 to max do begin
    FSOUND_Stream_GetTagField(Result,i,TagType,TagName,pointer(TagValue),integer(@TagLength));
    if TagName='ARTIST' then settag(Artist);
    if TagName='TITLE' then settag(Title);
    if TagName='ALBUM' then settag(Album);
    if TagName='TRACK' then settag(Track);
    if TagName='YEAR' then settag(Year);
    if TagName='GENRE' then settag(Genre);
    if TagName='COMMENT' then settag(Comment);
    if Assigned(Dest) then begin
      if ord(TagType)<>PrevTagType then begin
        PrevTagType:=ord(TagType);
        Dest.Add('');
        case TagType of
          FSOUND_TAGFIELD_VORBISCOMMENT: Dest.Add('Vorbis-Kommentare:');
          FSOUND_TAGFIELD_ID3V1: Dest.Add('ID3v1-Tag:');
          FSOUND_TAGFIELD_ID3V2: Dest.Add('ID3v2-Tag:');
          FSOUND_TAGFIELD_ASF: Dest.Add('ASF-Header:');
        end;
      end;
      TransTagName:=TagName;
      if TransTagName='ARTIST' then TransTagName:='Interpret';
      if TransTagName='TITLE' then TransTagName:='Titel';
      if TransTagName='COMMENT' then TransTagName:='Kommentar';
      if TransTagName='ALBUM' then TransTagName:='Album';
      if TransTagName='TRACK' then TransTagName:='Album-Titelnummer';
      if TransTagName='YEAR' then TransTagName:='Jahr';
      if TransTagName='GENRE' then
        Dest.Add('Genre: '+TranslateGenre(TagValue))
      else
        Dest.Add(TransTagName+': '+TagValue);
    end;
  end;
  if max>=0 then
    Item.Title:=ApplyTemplate(Artist,Title,Album,Track,Year,Genre,Comment);
  if (Mode<>gsiKeepHandle) then FSOUND_Stream_Close(Result);
end;

end.
