unit LoadSave;
interface
uses SysUtils, Classes, INIFiles;

const ConfigFileName='kjmp3.ini';

procedure SaveSettings;
procedure LoadSettings;

implementation
uses Globals;

procedure WriteStrings(INI:TMemIniFile; List:TStrings; const Section:string);
var i,max:integer;
begin
  max:=List.Count;
  INI.WriteInteger(Section,'n',max);
  for i:=1 to max do
    INI.WriteString(Section,IntToStr(i),List[i-1]);
end;

procedure SaveSettings;
var INI:TMemIniFile;
begin
  try
    INI:=TMemIniFile.Create(ConfigFileName);
  except
    exit;
  end;
  with INI do begin
    WriteInteger('kjmp3','SortOrder',    Ord(SortOrder));
    WriteString ('kjmp3','TitleTemplate',TitleTemplate);
    WriteInteger('kjmp3','MaxVotings',   MaxVotings);
    WriteInteger('kjmp3','Fadeout',      Ord(Fadeout));
    WriteString ('kjmp3','SoundDevice',  SoundDevice);
    WriteInteger('kjmp3','SoundRate',    SoundRate);
    WriteInteger('kjmp3','SoundOutput',  SoundOutput);
    WriteStrings(INI,IncludeDirs,'IncludeDirs');
    WriteStrings(INI,ExcludeDirs,'ExcludeDirs');
    UpdateFile;
    Free;
  end;
end;

procedure ReadStrings(INI:TMemIniFile; List:TStrings; const Section:string);
var i,max:integer; s:string;
begin
  List.Clear;
  if not INI.SectionExists(Section) then exit;
  max:=INI.ReadInteger(Section,'n',0);
  for i:=1 to max do begin
    s:=INI.ReadString(Section,IntToStr(i),'');
    if s<>'' then List.Add(s);
  end;
end;

procedure LoadSettings;
var INI:TMemIniFile;
begin
  try
    INI:=TMemIniFile.Create(ConfigFileName);
  except
    exit;
  end;
  with INI do begin
    CaseSensitive:=false;
    SortOrder:=TSortOrder(ReadInteger('kjmp3','SortOrder',Ord(soTitle)));
    TitleTemplate:=ReadString ('kjmp3','TitleTemplate',   '$artist  $title $comment');
    MaxVotings   :=ReadInteger('kjmp3','MaxVotings',      2);
    Fadeout:=TFadeoutType(ReadInteger('kjmp3','Fadeout',Ord(foSound)));
    SoundDevice  :=ReadString ('kjmp3','SoundDevice',     '');
    SoundRate    :=ReadInteger('kjmp3','SoundRate',       44100);
    SoundOutput  :=ReadInteger('kjmp3','SoundOutput',     -1);
    ReadStrings(INI,IncludeDirs,'IncludeDirs');
    ReadStrings(INI,ExcludeDirs,'ExcludeDirs');
    Free;
  end;
end;

end.
