{   MPEGinfo library for Pascal  --  extracts header and ID3 information
                                     from MP3 files
    Copyright (C) 1999  Martin Fiedler  <martin.fiedler@gmx.net>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.                         }

unit MP3Tags;

interface

type tagbuffer=array[0..127]of char;

type mpeghdr=record
               b:array[0..3]of byte; { the 4 original header bytes }
               valid:boolean;       { valid MPEG file? }
               version:byte;        { version numer: st_version[version] }
               layer:byte;          { Audio Layer No. }
               protected:boolean;   { CRC error protection? }
               bitrate:word;        { bitrate in kbps }
               samplerate:word;     { samplerate in Hz }
               padding:boolean;     { padding bit (used internally) }
               private:boolean;     { private bit (no idea what it means) }
               channels:byte;       { channel mode: st_channels[channels] }
               js_is:boolean;       { Joint Stereo: Intensity Stereo coding? }
               js_ms:boolean;       { Joint Stereo: M/S-coding? }
               copyrighted:boolean; { copyrightened material? }
               original:boolean;    { original material? }
               emphasis:byte;       { emphasis: st_emphasis[emphasis] }
               framesize:word;      { byte size of a frame
                                      (sometimes inaccurate) }
               filesize:longint;    { file size }
               songlen:longint;     { song length }
             end;
     id3info=record
               valid:boolean;       { file contains ID3 info? }
               title:string[30];    { song title }
               artist:string[30];   { artist }
               album:string[30];    { album name }
               year:string[4];      { year }
               comment:string[29];  { comment }
               cdtrack:byte;        { original CD track (ID3v1.1) }
               genre:byte;          { genre: st_genre(genre) }
             end;

const last_known_genre=79;
      gdata:array[0..last_known_genre]of string[31]=
('Blues','Classic Rock','Country','Dance','Disco','Funk','Grunge','Hip-Hop',
'Jazz','Metal','New Age','Oldies','Other','Pop','R&B','Rap','Reggae','Rock',
'Techno','Industrial','Alternative','Ska','Death Metal','Pranks',
'Soundtrack','Euro-Techno','Ambient','Trip-Hop','Vocal','Jazz+Funk','Fusion',
'Trance','Classical','Instrumental','Acid','House','Game','Sound Clip',
'Gospel','Noise','AlternRock','Bass','Soul','Punk','Space','Meditative',
'Instrumental Pop','Instrumental Rock','Ethnic','Gothic','Darkwave',
'Techno-Industrial','Electronic','Pop-Folk','Eurodance','Dream',
'Southern Rock','Comedy','Cult','Gangsta','Top 40','Christian Rap',
'Pop/Funk','Jungle','Native American','Cabaret','New Wave','Psychadelic',
'Rave','Showtunes','Trailer','Lo-Fi','Tribal','Acid Punk','Acid Jazz',
'Polka','Retro','Musical','Rock & Roll','Hard Rock');

const st_version:array[0..3]of string[3]=('2.5','---','2.0','1.0');
const st_channels:array[0..3]of string[13]=
                             ('Stereo','Joint Stereo','Dual Channel','Mono');
const st_emphasis:array[0..3]of string[11]=
                                ('none','50/15 s','invalid','CCITT J.17');

function st_rate(var inrate:word):string;
function st_genre(b:byte):string;

procedure valhdr(b1,b2,b3,b4:byte;var h:mpeghdr);
procedure valtag(var d:tagbuffer;var t:id3info);

function ReadMP3Info(const filename:string;
                     var header:mpeghdr;var tag:id3info):boolean;
function WriteID3Info(filename:string;tag:id3info):boolean;


implementation

function st_rate(var inrate:word):string;
var s:string[3];
begin
  case inrate of
    0:st_rate:='VBR';
    65535:st_rate:='ungltige Bitrate';
  else
    str(inrate,s);st_rate:=s+' kbps';
  end;
end;

function st_genre(b:byte):string;
var s:string[3];
begin
  case b of
    0..last_known_genre:st_genre:=gdata[b];
    255:st_genre:='< unbekannt >';
  else
    str(b,s);
    st_genre:='< unbekannt (#'+s+') >';
  end;
end;

procedure valhdr(b1,b2,b3,b4:byte;var h:mpeghdr);
const bitrates:array[1..2,1..3,0..15]of word=
       (((0, 32, 64, 96,128,160,192,224,256,288,320,352,384,416,448,65535),
         (0, 32, 48, 56, 64, 80, 96,112,128,160,192,224,256,320,384,65535),
         (0, 32, 40, 48, 56, 64, 80, 96,112,128,160,192,224,256,320,65535)),
        ((0, 32, 64, 96,128,160,192,224,256,288,320,352,384,416,448,65535),
         (0, 32, 48, 56, 64, 80, 96,112,128,160,192,224,256,320,284,65535),
         (0,  8, 16, 24, 32, 54, 80, 56, 64,128,160,112,128,256,320,65535)));
      brversconv:array[0..3]of byte=(2,1,2,1);
      brlayconv:array[1..4]of byte=(1,2,3,3);
      samprates:array[1..3,0..3]of word=((44100,48000,32000,0),
                                         (22050,24000,16000,0),
                                         (11025,12000, 8000,0));
      srversconv:array[0..3]of byte=(3,1,2,1);
      lengthfactor:array[1..2,1..2]of longint=((48000,144000),(24000,72000));
      fllayconv:array[0..3]of byte=(1,2,2,2);
      framelens:array[0..3]of integer=(576,1152,576,1152);
var l2:longint;
begin
  with h do begin
    valid:=((b1=$FF) AND (b2 AND $E0=$E0));
    version:=b2 SHR 3 AND 3;
    layer:=4-(b2 SHR 1 AND 3);
    protected:=not(boolean(b2 AND 1));
    bitrate:=bitrates[brversconv[version],brlayconv[layer],b3 SHR 4];
    samplerate:=samprates[srversconv[version],b3 SHR 2 AND 3];
    padding:=boolean(b3 SHR 1 AND 1);
    private:=boolean(b3 AND 1);
    channels:=b4 SHR 6;
    js_is:=boolean(b4 SHR 4 AND 1);
    js_ms:=boolean(b4 SHR 5 AND 1);
    copyrighted:=boolean(b4 SHR 3 AND 1);
    original:=boolean(b4 SHR 2 AND 1);
    emphasis:=b4 AND 3;
    l2:=lengthfactor[brversconv[version],fllayconv[layer]];
    if valid then
    framesize:=longint(longint(l2*longint(bitrate)) DIV longint(samplerate))
               +byte(padding);
    if valid then
      songlen:=round(filesize/(framesize+1)*framelens[version]/samplerate*1000.0);
  end;
end;

procedure valtag(var d:tagbuffer;var t:id3info);
var b:byte;
begin
  with t do begin
    valid:=((d[0]='T') AND (d[1]='A') AND (d[2]='G'));
    title[0]:=#30;move(d[3],title[1],30);b:=pos(#0,title);
      if b>0 then title[0]:=char(b-1) else begin while (length(title)>0)
      AND (title[length(title)]=#32) do dec(title[0]);end;
    artist[0]:=#30;move(d[33],artist[1],30);b:=pos(#0,artist);
      if b>0 then artist[0]:=char(b-1) else begin while (length(artist)>0)
      AND (artist[length(artist)]=#32) do dec(artist[0]);end;
    album[0]:=#30;move(d[63],album[1],30);b:=pos(#0,album);
      if b>0 then album[0]:=char(b-1) else begin while (length(album)>0)
      AND (album[length(album)]=#32) do dec(album[0]);end;
    year[0]:=#4;move(d[93],year[1],4);b:=pos(#0,year);
      if b>0 then year[0]:=char(b-1) else begin while (length(year)>0)
      AND (year[length(year)]=#32) do dec(year[0]);end;
    comment[0]:=#29;move(d[97],comment[1],29);b:=pos(#0,comment);
      if b>0 then comment[0]:=char(b-1) else begin while (length(comment)>0)
      AND (comment[length(comment)]=#32) do dec(comment[0]);end;
    cdtrack:=byte(d[126]);
    genre:=byte(d[127]);
  end;
end;

function readmp3info(const filename:string;
                     var header:mpeghdr;var tag:id3info):boolean;
var w:integer;li:longint;f:file;buffer:tagbuffer; ofm:byte;
begin
  readmp3info:=false;header.valid:=false;tag.valid:=false;
  assign(f,filename);
  ofm:=FileMode; FileMode:=0;
  {$I-} reset(f,1); {$I+}
  FileMode:=ofm;
  if ioresult<>0 then exit;
  header.filesize:=filesize(f);
  with header do begin
    blockread(f,b[0],4,w);
    valhdr(b[0],b[1],b[2],b[3],header);
  end;
  li:=filesize(f)-128;
  {$I-} if li>0 then seek(f,li); {$I+}
  if ioresult<>0 then begin close(f);exit;end;
  blockread(f,buffer,128,w);valtag(buffer,tag);
  if tag.valid then dec(header.filesize,128);
  close(f);
  readmp3info:=true;
end;

function writeid3info(filename:string;tag:id3info):boolean;
var f:file;li:longint;header:mpeghdr;buffer:tagbuffer;b:byte;w:integer;
begin
  writeid3info:=false;header.valid:=false;
  assign(f,filename);
  {$I-} reset(f,1); {$I+}
  if ioresult<>0 then exit;
  with header do begin
    blockread(f,b[0],1,w);blockread(f,b[1],1,w);blockread(f,b[2],1,w);
    blockread(f,b[3],1,w);valhdr(b[0],b[1],b[2],b[3],header);
  end;
  header.filesize:=filesize(f);
  li:=filesize(f)-128;
  {$I-} if li>0 then seek(f,li); {$I+}
  if ioresult<>0 then begin close(f);exit;end;
  blockread(f,buffer,128,w);
  if ((buffer[0]='T') AND (buffer[1]='A') AND (buffer[2]='G')) then begin
    li:=filesize(f)-128;
    {$I-} if li>0 then seek(f,li); {$I+}
    if ioresult<>0 then begin close(f);exit;end;
  end;
  fillchar(buffer,128,0);
  buffer[0]:='T';buffer[1]:='A';buffer[2]:='G';
  b:=length(tag.title);if b>0 then move(tag.title[1],buffer[3],b);
  b:=length(tag.artist);if b>0 then move(tag.artist[1],buffer[33],b);
  b:=length(tag.album);if b>0 then move(tag.album[1],buffer[63],b);
  b:=length(tag.year);if b>0 then move(tag.year[1],buffer[93],b);
  b:=length(tag.comment);if b>0 then move(tag.comment[1],buffer[97],b);
  buffer[126]:=char(tag.cdtrack);
  buffer[127]:=char(tag.genre);
  blockwrite(f,buffer,128,w);
  close(f);writeid3info:=true;
end;

end.
