unit PLMan;
interface

const NotRated=$31337;

type PPlaylistItem=^TPlaylistItem;
     TPlaylistItem=record
                     MyIndex,RefPrev,RefNext:integer;
                     FileName:string;
                     FileDate,FileSize:integer;
                     Length:integer; sLength:string[7];
                     Title:string;
                     Rating,PlayCount:integer;
                     sRating:string[15];
                   end;

type TPlaylistStats=record
                      Count:integer;
                      TotalTime:int64;
                      TotalSize:int64;
                      TotalPlayCount:integer;
                      TotalRatings:integer;
                      cntGood,cntZero,cntBad,cntUnrated:integer;
                    end;

function EmptyPlaylistItem:TPlaylistItem;
procedure ClearPlaylistItem(var Item:TPlaylistItem);
procedure AdjustStrings(var Item:TPlaylistItem);

function TimeToString(Time:integer):string;

type TPlaylist=class
                 Data:array of TPlaylistItem;
                 FCount:integer;
                 FMaxRating:integer;
                 function GetItemPtr(Index:integer):PPlaylistItem;
                 function AllocItem:integer;
               public
                 property HighestRating:integer read FMaxRating;
                 property Count:integer read FCount;
                 property Items[Index:integer]:PPlaylistItem read GetItemPtr; default;
                 function Add(const Item:TPlaylistItem):integer;
                 function Find(const Item:TPlaylistItem):integer;
                 function GetNextItem:PPlaylistItem;
                 procedure SaveToFile(const FileName:string);
                 procedure LoadFromFile(const FileName:string);
                 procedure Clear;
                 function GetTotalTime:int64;
                 property TotalTime:int64 read GetTotalTime;
                 function GetStats:TPlaylistStats;
                 constructor Create;
                 constructor CreateFromPlaylist(PL:TPlaylist);
                 destructor Destroy; override;
               end;

function RateItem(const Item:TPlaylistItem):real;

implementation
uses SysUtils;

const PlaylistGranularity=128;

function EmptyPlaylistItem:TPlaylistItem;
begin ClearPlaylistItem(Result); end;

procedure ClearPlaylistItem(var Item:TPlaylistItem);
begin with Item do begin
  MyIndex:=-1; RefPrev:=-1; RefNext:=-1;
  FileName:=''; FileDate:=0; FileSize:=0;
  Length:=0; Title:=''; Rating:=NotRated; PlayCount:=0;
  AdjustStrings(Item);
end; end;

function TimeToString(Time:integer):string;
begin
  if Time=0 then Result:=#151 else
  Result:=#32+IntToStr(Time DIV 60000)+':'+
          char(48+Time DIV 10000 MOD 6)+char(48+Time DIV 1000 MOD 10);
end;

procedure AdjustStrings(var Item:TPlaylistItem);
begin with Item do begin
  sLength:=TimeToString(Length);
  if Rating=NotRated then sRating:='---'
  else if Rating=0   then sRating:='0/'+IntToStr(PlayCount)
  else if Rating>0   then sRating:='+'+IntToStr(Rating)+'/'+IntToStr(PlayCount)
                     else sRating:=IntToStr(Rating)+'/'+IntToStr(PlayCount);
end; end;

constructor TPlaylist.Create;
begin
  inherited Create;
  SetLength(Data,PlaylistGranularity);
  Clear;
end;

constructor TPlaylist.CreateFromPlaylist(PL:TPlaylist);
begin
  inherited Create;
  Data:=PL.Data;
  FCount:=PL.FCount;
  FMaxRating:=PL.FMaxRating;
end;

destructor TPlaylist.Destroy;
begin
  Finalize(Data);
  inherited Destroy;
end;

function TPlaylist.GetItemPtr(Index:integer):PPlaylistItem;
begin
  if (Index>=0) AND (Index<FCount) then Result:=@Data[Index] else Result:=nil;
end;

function TPlaylist.AllocItem:integer;
begin
  Result:=FCount;
  inc(FCount);
  if FCount>length(Data) then
    SetLength(Data,FCount+PlaylistGranularity);
end;

function TPlaylist.Add(const Item:TPlaylistItem):integer;
var Last,LastDir:integer;
begin
//writeln('Adding ',Item.FileName);
  if FCount>0 then Result:=0 else Result:=-1;
  Last:=-1; LastDir:=0;
  while Result>=0 do
    if Item.FileName>Data[Result].FileName then begin
      Last:=Result; LastDir:=1;
      Result:=Data[Result].RefNext;
    end else if Item.FileName=Data[Result].FileName then
      exit
    else begin
      Last:=Result; LastDir:=0;
      Result:=Data[Result].RefPrev;
    end;
  Result:=AllocItem();
  Data[Result]:=Item;
  with Data[Result] do begin
    MyIndex:=Result;
    RefPrev:=-1;
    RefNext:=-1;
  end;
  if Last>=0 then begin
    if LastDir>0 then Data[Last].RefNext:=Result
                 else Data[Last].RefPrev:=Result;
  end;
  if Item.Rating<>NotRated then begin
    Last:=abs(Item.Rating);
    if Last>FMaxRating then FMaxRating:=Last;
  end;
end;

function TPlaylist.Find(const Item:TPlaylistItem):integer;
begin
  if FCount>0 then Result:=0 else Result:=-1;
  while Result>=0 do
    if Item.FileName=Data[Result].FileName then
      exit
    else if Item.FileName>Data[Result].FileName then
      Result:=Data[Result].RefNext
    else
      Result:=Data[Result].RefPrev;
end;

procedure TPlaylist.Clear;
begin
  FCount:=0; FMaxRating:=1;
end;

function RateItem(const Item:TPlaylistItem):real;
begin with Item do begin
  if Rating=NotRated then Result:=NotRated
  else if Rating=0 then Result:=1.0
                   else Result:=1.0+Rating/PlayCount;
  if Result<0.0 then Result:=0.0;
end; end;

function TPlaylist.GetNextItem:PPlaylistItem;
var Ratings:array of real;
    Index:integer; RPos:real;
begin
  Result:=nil; if FCount<1 then exit;
  SetLength(Ratings,FCount);
  RPos:=0.0;
  for Index:=0 to FCount-1 do begin
    RPos:=RPos+RateItem(Data[Index]);
    Ratings[Index]:=RPos;
  end;
  RPos:=random*RPos;
  Index:=0; while (Index<FCount) AND (Ratings[Index]<RPos) do inc(Index);
  Result:=@Data[Index];
end;

procedure TPlaylist.SaveToFile(const FileName:string);
var Buf:array of integer; BPos:integer;
const BufferGranularity=1024*1024;
  procedure Resize(s:integer); begin
    if s>length(Buf) then
      SetLength(Buf,s+BufferGranularity);
  end;
  procedure PutInteger(i:integer); begin
    Resize(BPos+1);
    Buf[BPos]:=i;
    inc(BPos);
  end;
  procedure PutString(const s:string); var cnt:integer; begin
    cnt:=1+length(s) DIV 4;
    Resize(BPos+cnt);
    Buf[BPos+cnt-1]:=0;
    Move(s[1],Buf[BPos],length(s));
    inc(BPos,cnt);
  end;
var i:integer; f:file;
begin
  SetLength(Buf,BufferGranularity);
  BPos:=0;
  PutInteger(FCount);
  for i:=0 to FCount-1 do
    with Data[i] do begin
      PutInteger(RefPrev); PutInteger(RefNext);
      PutString(FileName);
      PutInteger(FileDate); PutInteger(FileSize);
      PutString(Title); PutInteger(Length);
      PutInteger(Rating); PutInteger(PlayCount);
    end;
  AssignFile(f,FileName);
  {$I-} Rewrite(f,4); {$I+}
  if IOresult<>0 then exit;
  BlockWrite(f,Buf[0],BPos);
  CloseFile(f);
end;

procedure TPlaylist.LoadFromFile(const FileName:string);
var Buf:array of integer; BPos:integer;
  function GetInteger:integer; begin
    Result:=Buf[BPos]; inc(BPos);
  end;
  function GetString:string; var b:PChar; begin
    b:=@Buf[BPos]; Result:=b;
    inc(BPos,1+length(Result) DIV 4);
  end;
var i:integer; f:file;
begin
  AssignFile(f,FileName);
  {$I-} Reset(f,4); {$I+}
  if IOresult<>0 then exit;
  SetLength(Buf,FileSize(f));
  BlockRead(f,Buf[0],length(Buf));
  CloseFile(f);
  if length(Buf)<1 then exit;
  BPos:=0;
  FCount:=GetInteger; FMaxRating:=1;
  SetLength(Data,FCount);
  for i:=0 to FCount-1 do
    with Data[i] do begin
      ClearPlaylistItem(Data[i]);
      RefPrev:=GetInteger; RefNext:=GetInteger;
      FileName:=GetString;
      FileDate:=GetInteger; FileSize:=GetInteger;
      Title:=GetString; Length:=GetInteger;
      Rating:=GetInteger; PLayCount:=GetInteger;
      AdjustStrings(Data[i]);
    end;
end;

function TPlaylist.GetTotalTime:int64;
var i:integer;
begin
  Result:=0;
  for i:=0 to (FCount-1) do
    inc(Result,Data[i].Length);
  Result:=(Result+500) DIV 1000;
end;

function TPlaylist.GetStats:TPlaylistStats;
var i:integer;
begin with Result do begin
  FillChar(Result,sizeof(Result),0);
  for i:=0 to (FCount-1) do
    with Data[i] do begin
      inc(Count);
      inc(TotalTime,Length);
      inc(TotalSize,FileSize);
      inc(TotalPlayCount,PlayCount);
      if Rating<>NotRated then begin
        inc(TotalRatings,abs(Rating));
        if Rating>0 then inc(cntGood)
        else if Rating<0 then inc(cntBad)
        else inc(cntZero);
      end else
        inc(cntUnrated);
    end;
  TotalTime:=(TotalTime+500) DIV 1000;
end; end;

end.
