unit Playlist;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, PLMan, fmod, fmodtypes, ExtCtrls, Buttons, ComCtrls,
  Menus;

const PlaylistFile='kjmp3.pl';

type
  TPlaylistForm = class(TForm)
    PDisp: TPanel;
    GInfo: TGroupBox;
    LTitle: TLabel;
    PPos0: TPanel;
    LElapsed: TLabel;
    LTotal: TLabel;
    PPos1: TPanel;
    UpdateTimer: TTimer;
    PList: TPanel;
    PHeaders: TPanel;
    HTitle: TPanel;
    LList: TListBox;
    HRating: TPanel;
    BVoteDown: TSpeedButton;
    BSkip: TSpeedButton;
    BVoteUp: TSpeedButton;
    BPause: TSpeedButton;
    BMenu: TSpeedButton;
    MInfo: TMemo;
    Stats: TStatusBar;
    LMiniTitle: TLabel;
    PopupMenu: TPopupMenu;
    MQuit: TMenuItem;
    MSettings: TMenuItem;
    N1: TMenuItem;
    MFind: TMenuItem;
    MFindNext: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure LListDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure LListDblClick(Sender: TObject);
    procedure UpdateTimerTimer(Sender: TObject);
    procedure PPosMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PPosMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure BVoteClick(Sender: TObject);
    procedure VirtualButtonMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure VirtualButtonMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure HClick(Sender: TObject);
    procedure LListClick(Sender: TObject);
    procedure BPauseClick(Sender: TObject);
    procedure StatsClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure MQuitClick(Sender: TObject);
    procedure BMenuClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure MSettingsClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure LListKeyPress(Sender: TObject; var Key: Char);
    procedure MFindClick(Sender: TObject);
    procedure MFindNextClick(Sender: TObject);
  private
    { Private declarations }
    LastSeekPoint:integer;
    FindStr:string;
    procedure FindNextEx(Start:integer);
  public
    { Public declarations }
    procedure InitSound;
    procedure PlayItem(Index:integer);
    procedure Vote(RatingDiff:integer);
    procedure NextTrack;
    procedure UpdateStats;
    procedure Find;
    procedure FindNext;
  end;

var
  PlaylistForm: TPlaylistForm;

implementation
uses Update, Globals, GetInfo, Settings, LoadSave;

const BarSize=96;
      BarHalfSize=BarSize DIV 2;
      BarOffset=BarSize+48;
      BarCenter=BarOffset-BarHalfSize;
      NotRatedText='nicht bewertet';

{$R *.dfm}

procedure TPlaylistForm.InitSound;
var i:integer;
begin
  FSOUND_SetOutput(TFSoundOutputTypes(SoundOutput));
  for i:=FSOUND_GetNumDrivers()-1 downto 0 do
    if FSOUND_GetDriverName(i)=SoundDevice then begin
      FSOUND_SetDriver(i);
      break;
    end;
  FSOUND_Init(SoundRate,1,FSOUND_INIT_GLOBALFOCUS);
end;

procedure TPlaylistForm.FormCreate(Sender: TObject);
begin
  randomize;
  DecimalSeparator:='.';
  HRating.Width:=BarOffset+GetSystemMetrics(SM_CXVSCROLL);
//AllocConsole;
  LoadDefaults;
  LoadSettings;
  MainPL:=TPlaylist.Create;
  MainPL.LoadFromFile(PlaylistFile);
  InitSound;
  UpdatePlaylist(false);
  UpdateStats;
  NextTrack;
  FindStr:='';
end;

procedure TPlaylistForm.FormDestroy(Sender: TObject);
begin
  MainPL.SaveToFile(PlaylistFile);
  if Assigned(TheStream) then FSOUND_Stream_Close(TheStream);
  FSOUND_Close();
end;

procedure TPlaylistForm.LListDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var x:integer; Clip:TRect;
begin with LList.Canvas,MainPL[Order[Index]]^ do begin
  FillRect(Rect);
  Clip:=Rect; dec(Clip.Right,BarOffset+10);
  TextRect(Clip,Rect.Left+3,Rect.Top+1,Title);
  TextOut(Rect.Right-5-BarOffset-TextWidth(sLength),Rect.Top+1,sLength);
  if Rating=NotRated then
    TextOut(Rect.Right-(BarOffset+TextWidth(NotRatedText)) DIV 2,
            Rect.Top+1,NotRatedText)
  else begin
    TextOut(Rect.Right-3-TextWidth(sRating),Rect.Top+1,sRating);
    Pen.Color:=$C0C0C0;
    Brush.Color:=$FFFFFF;
    Rectangle(Rect.Right-BarOffset,Rect.Top+1,
              Rect.Right-BarOffset+BarSize,Rect.Bottom-1);
    if Rating=0 then begin
      MoveTo(Rect.Right-BarCenter,Rect.Top+2);
      LineTo(Rect.Right-BarCenter,Rect.Bottom-1);
    end else if Rating>0 then begin
      x:=PlayCount; if x<1 then x:=1;
      x:=Rect.Right-BarCenter+1+Rating*BarHalfSize DIV x;
      if x>Rect.Right-BarOffset+BarSize-1 then x:=Rect.Right-BarOffset+BarSize-1;
      Brush.Color:=$00A000;
      FillRect(Classes.Rect(Rect.Right-BarCenter,Rect.Top+2,x,Rect.Bottom-2));
    end else begin
      x:=PlayCount; if x<1 then x:=1;
      x:=Rect.Right-BarCenter+Rating*BarHalfSize DIV x;
      if x<Rect.Right-BarOffset+1 then x:=Rect.Right-BarOffset+1;
      Brush.Color:=$0000A0;
      FillRect(Classes.Rect(x,Rect.Top+2,Rect.Right-BarCenter+1,Rect.Bottom-2));
    end;
  end;
end; end;

procedure TPlaylistForm.LListDblClick(Sender: TObject);
var Index:integer;
begin
  Index:=LList.ItemIndex;
  if Index<0 then exit;
  Index:=Order[Index];
  PlayItem(Index);
end;

procedure TPlaylistForm.PlayItem(Index:integer);
begin
  UpdateTimer.Enabled:=false;
  if Assigned(TheStream) then FSOUND_Stream_Close(TheStream);
  PlayingNow:=MainPL[Index];
  GInfo.Caption:='Datei-Informationen';
  MInfo.Text:='Lese Informationen ...';
  GInfo.Repaint;
  TheStream:=GetSongInfo(PlayingNow^,'',MInfo.Lines,gsiKeepHandle);
  UpdateStats;
  LList.Invalidate;
  with PlayingNow^ do begin
    StreamLength:=Length;
    LTotal.Caption:=TimeToString(Length);
    LTitle.Caption:=Title;
    LMiniTitle.Caption:=Title;
    Caption:='kjmp3 :: '+Title;
    Application.Title:=Title;
  end;
  LastSeekPoint:=-maxint; AlreadyVoted:=0;
  if not Paused then
    TheChannel:=FSOUND_Stream_Play(FSOUND_FREE,TheStream);
  UpdateTimerTimer(nil);
  UpdateTimer.Enabled:=true;
end;

procedure TPlaylistForm.UpdateTimerTimer(Sender: TObject);
var Time,PLen:integer;
begin
  if Assigned(TheStream) then Time:=FSOUND_Stream_GetTime(TheStream) else Time:=0;
  LElapsed.Caption:=TimeToString(Time);
  PLen:=Time*(PPos0.Width-2) DIV StreamLength;
  PPos1.Visible:=(PLen>0);
  PPos1.Width:=PLen;
  if Time>=StreamLength then Vote(0);
end;

procedure TPlaylistForm.PPosMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  PPosMouseMove(Sender,Shift+[ssLeft],X,Y);
end;

procedure TPlaylistForm.PPosMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  if not Assigned(TheStream) then exit;
  if not (ssLeft IN Shift) then exit;
  if Sender=PPos0 then dec(X);
  if X=LastSeekPoint then exit;
  LastSeekPoint:=X;
  FSOUND_Stream_SetTime(TheStream,X*StreamLength DIV (PPos0.Width-2));
end;

procedure TPlaylistForm.Vote(RatingDiff:integer);
begin
  if not Assigned(PlayingNow) then exit;
  if (RatingDiff<>0) AND (AlreadyVoted>=MaxVotings) then exit;
  with PlayingNow^ do begin
    if Rating=NotRated then Rating:=RatingDiff else inc(Rating,RatingDiff);
    if AlreadyVoted<=0 then inc(PlayCount);
  end;
  inc(AlreadyVoted);
  AdjustStrings(PlayingNow^);
  LList.Invalidate;
  if RatingDiff<1 then NextTrack;
end;

procedure TPlaylistForm.BVoteClick(Sender: TObject);
begin
  Vote((Sender as TControl).Tag);
end;

procedure TPlaylistForm.NextTrack;
var i:integer; Item:PPlaylistItem;
begin
  Item:=MainPL.GetNextItem;
  if not Assigned(Item) then exit;
  PlayItem(Item^.MyIndex);
  for i:=0 to High(Order) do
    if Order[i]=PlayingNow^.MyIndex then
      LList.ItemIndex:=i;
end;

procedure TPlaylistForm.VirtualButtonMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  (Sender as TPanel).BevelOuter:=bvLowered;
end;

procedure TPlaylistForm.VirtualButtonMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  (Sender as TPanel).BevelOuter:=bvRaised;
end;

procedure TPlaylistForm.HClick(Sender: TObject);
var Index,i:integer;
begin
  Index:=LList.ItemIndex;
  if Index>=0 then Index:=Order[Index];
  SetSortOrder(TSortOrder((Sender as TControl).Tag));
  LList.Invalidate;
  if Index<0 then exit;
  for i:=0 to High(Order) do
    if Order[i]=Index then
      LList.ItemIndex:=i;
end;

procedure TPlaylistForm.LListClick(Sender: TObject);
var Index:integer;
begin
  Index:=LList.ItemIndex;
  if Index<0 then exit;
  Index:=Order[Index];
  GInfo.Caption:='Datei-Informationen';
  MInfo.Text:='Lese Informationen ...';
  GInfo.Repaint;
  GetSongInfo(MainPL[Index]^,'',MInfo.Lines,gsiThorough);
  UpdateStats;
  LList.Invalidate;
end;

procedure TPlaylistForm.BPauseClick(Sender: TObject);
begin
  Paused:=not(Paused);
  if Paused then begin
    UpdateTimer.Enabled:=false;
    UpdateTimerTimer(nil);
    PausePos:=FSOUND_Stream_GetTime(TheStream);
    FSOUND_Stream_Stop(TheStream);
    TheChannel:=0;
  end else begin
    TheChannel:=FSOUND_Stream_Play(FSOUND_FREE,TheStream);
    FSOUND_Stream_SetTime(TheStream,PausePos);
    UpdateTimer.Enabled:=true;
  end;
  BPause.Down:=Paused;
end;

procedure TPlaylistForm.UpdateStats;
begin
  Stats.SimpleText:=#32+IntToStr(MainPL.Count)+' Titel, ca. '+
                    ApproxTime(MainPL.TotalTime);
end;

procedure TPlaylistForm.StatsClick(Sender: TObject);
var s:string;
const
  KiB:int64=1024;
  MiB:int64=1048576;
  GiB:int64=1073741824;
  TiB:int64=1099511627776;
  DataCD:int64=700*1024*1024;
  AudioCD:int64=80*60;
begin with MInfo.Lines,MainPL.GetStats do begin
  GInfo.Caption:='Gesamtstatistik';
  Clear;
  Add(IntToStr(Count)+' Titel in der Playlist');
  Add('Positiv bewertet: '+IntToStr(cntGood)+' ('+
      FloatToStrF(100.0*cntGood/Count,ffFixed,7,1)+'%)');
  Add('Mittelmig bewertet: '+IntToStr(cntZero)+' ('+
      FloatToStrF(100.0*cntZero/Count,ffFixed,7,1)+'%)');
  Add('Negativ bewertet: '+IntToStr(cntBad)+' ('+
      FloatToStrF(100.0*cntBad/Count,ffFixed,7,1)+'%)');
  Add('Unbewertet: '+IntToStr(cntUnrated)+' ('+
      FloatToStrF(100.0*cntUnrated/Count,ffFixed,7,1)+'%)');
  Add('Bewertungen abgegeben: '+IntToStr(TotalRatings));
  Add('Titel abgespielt: '+IntToStr(TotalPlayCount));

  Add('');
  Add('Gesamtspielzeit: ca. '+ApproxTime(TotalTime));
  Add('  = '+StringReplace(Format('%d:%2d:%2d',
      [TotalTime DIV 3600,TotalTime DIV 60 MOD 60,TotalTime MOD 60]),
      #32,'.',[rfReplaceAll]));
  TotalTime:=(TotalTime+AudioCD-1) DIV AudioCD;
  Add('  = '+IntToStr(TotalTime)+' Audio-CDs (ca. '+
      FloatToStrF(TotalTime*0.135,ffFixed,7,1)+' cm)');

  Add('');
       if TotalSize<KiB then s:=IntToStr(TotalSize)+' Bytes'
  else if TotalSize<MiB then s:=FloatToStrF(TotalSize/KiB,ffFixed,7,1)+' KiB'
  else if TotalSize<GiB then s:=FloatToStrF(TotalSize/MiB,ffFixed,7,1)+' MiB'
  else if TotalSize<TiB then s:=FloatToStrF(TotalSize/GiB,ffFixed,7,1)+' GiB'
                        else s:=FloatToStrF(TotalSize/TiB,ffFixed,7,1)+' TiB';
  Add('Gesamtdatengre: '+s);
  Add('  = '+ThousandDots(TotalSize)+' Bytes');
  TotalSize:=(TotalSize+DataCD-1) DIV DataCD;
  Add('  = '+IntToStr(TotalSize)+' Daten-CDs (ca. '+
      FloatToStrF(TotalSize*0.135,ffFixed,7,1)+' cm)');
end; end;

procedure TPlaylistForm.FormResize(Sender: TObject);
var Mini:boolean;
begin
  LList.Invalidate;
  GInfo.Visible:=(ClientHeight>=160);
  Mini:=(ClientWidth<320);
  LTitle.Visible:=not(Mini);
  LMiniTitle.Visible:=Mini;
end;

procedure TPlaylistForm.MQuitClick(Sender: TObject);
begin
  Close;
end;

procedure TPlaylistForm.BMenuClick(Sender: TObject);
begin
  with ClientToScreen(Point(ClientWidth-1,BMenu.Top+BMenu.Height)) do
    PopupMenu.Popup(X,Y);
end;

procedure TPlaylistForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
var vol:integer;
begin
  if Fadeout=foNone then exit;
  if TheChannel=0 then exit;
  vol:=255;
  if Fadeout=foVisual then begin
    AlphaBlend:=True;
    Repaint;
  end;
  while vol>0 do begin
    dec(vol,4);
    FSOUND_SetVolume(TheChannel,vol);
    if Fadeout=foVisual then AlphaBlendValue:=vol;
    Sleep(10);
  end;
  FSOUND_SetVolume(TheChannel,0);
  if Fadeout=foVisual then AlphaBlendValue:=0;
end;

procedure TPlaylistForm.MSettingsClick(Sender: TObject);
begin
  SettingsForm.ShowModal;
end;

procedure TPlaylistForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  case Key of
    ^P:SettingsForm.ShowModal;
    ^F:Find;
  end;
end;

procedure TPlaylistForm.Find;
var s:string;
begin
  s:=FindStr;
  if not InputQuery('kjmp3::Titel suchen','Teil des gesuchten Titelnamens eingeben:',s) then exit;
  FindStr:=LowerCase(s);
  FindNextEx(-1);
end;

procedure TPlaylistForm.FindNext;
begin
  FindNextEx(LList.ItemIndex);
end;

procedure TPlaylistForm.FindNextEx(Start:integer);
var Index:integer;
begin
  if MainPL.Count<1 then exit;
  if Start<0 then Start:=MainPL.Count-1;
  Index:=(Start+1) MOD MainPL.Count;
  while Index<>Start do begin
    if pos(FindStr,LowerCase(MainPL[Order[Index]]^.Title))>0 then begin
      LList.ItemIndex:=Index;
      ActiveControl:=LList;
      LListClick(nil);
      exit;
    end;
    Index:=(Index+1) MOD MainPL.Count;
  end;
  MessageDlg('Kein passender Titel gefunden!',mtInformation,[mbOK],0);
end;

procedure TPlaylistForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    VK_F3:FindNext;
  end;
end;

procedure TPlaylistForm.LListKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#13 then LListDblClick(nil);
end;

procedure TPlaylistForm.MFindClick(Sender: TObject);
begin
  Find;
end;

procedure TPlaylistForm.MFindNextClick(Sender: TObject);
begin
  FindNext;
end;

end.
