unit Settings;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, fmod, fmodtypes;

type
  TSettingsForm = class(TForm)
    Pager: TPageControl;
    PMisc: TTabSheet;
    BCancel: TButton;
    BOK: TButton;
    PDirs: TTabSheet;
    PDInclude: TPanel;
    LInclude: TLabel;
    DInclude: TListBox;
    BAddInclude: TButton;
    BDelInclude: TButton;
    PDExclude: TPanel;
    LExclude: TLabel;
    DExclude: TListBox;
    BAddExclude: TButton;
    BDelExclude: TButton;
    PDSplitter: TSplitter;
    LTitleTemplate: TLabel;
    ETitleTemplate: TEdit;
    LTitleExpl: TLabel;
    LMaxVotings: TLabel;
    EMaxVotings: TEdit;
    CMaxVotings: TUpDown;
    LFadeout: TLabel;
    CFadeout: TComboBox;
    LDevice: TLabel;
    CDevice: TComboBox;
    LRate: TLabel;
    CRate: TComboBox;
    COutput: TComboBox;
    LOutput: TLabel;
    procedure FormShow(Sender: TObject);
    procedure DIncludeClick(Sender: TObject);
    procedure DExcludeClick(Sender: TObject);
    procedure BAddIncludeClick(Sender: TObject);
    procedure BAddExcludeClick(Sender: TObject);
    procedure BDelIncludeClick(Sender: TObject);
    procedure BDelExcludeClick(Sender: TObject);
    procedure BOKClick(Sender: TObject);
    procedure SoundChange(Sender: TObject);
  private
    { Private declarations }
    ListModified:integer;
    SoundModified:boolean;
  public
    { Public declarations }
  end;

var
  SettingsForm: TSettingsForm;

implementation
uses Globals, DirSelect, Update, Playlist, LoadSave;

{$R *.dfm}

procedure TSettingsForm.FormShow(Sender: TObject);
var i,max:integer;
begin
  ListModified:=0; SoundModified:=false;
  DInclude.Items.Assign(IncludeDirs); DIncludeClick(nil);
  DExclude.Items.Assign(ExcludeDirs); DIncludeClick(nil);
  ETitleTemplate.Text:=TitleTemplate;
  CMaxVotings.Position:=MaxVotings;
  CFadeout.ItemIndex:=ord(Fadeout);
  max:=FSOUND_GetNumDrivers();
  CDevice.Items.Clear;
  for i:=0 to (max-1) do
    CDevice.Items.Add(FSOUND_GetDriverName(i));
  CDevice.ItemIndex:=FSOUND_GetDriver();
  CRate.Text:=IntToStr(FSOUND_GetOutputRate());
  case FSOUND_GetOutput() of
    FSOUND_OUTPUT_NOSOUND: COutput.ItemIndex:=1;
    FSOUND_OUTPUT_WINMM:   COutput.ItemIndex:=2;
    FSOUND_OUTPUT_DSOUND:  COutput.ItemIndex:=3;
    FSOUND_OUTPUT_A3D:     COutput.ItemIndex:=4;
    FSOUND_OUTPUT_ASIO:    COutput.ItemIndex:=5;
  else
    COutput.ItemIndex:=0;
  end;
end;

procedure TSettingsForm.BOKClick(Sender: TObject);
begin
  IncludeDirs.Assign(DInclude.Items);
  ExcludeDirs.Assign(DExclude.Items);
  if TitleTemplate<>ETitleTemplate.Text then begin
    TitleTemplate:=ETitleTemplate.Text;
    ListModified:=2;
  end;
  MaxVotings:=CMaxVotings.Position;
  Fadeout:=TFadeoutType(CFadeout.ItemIndex);
  if ListModified>0 then begin
    UpdatePlaylist(ListModified>1);
    PlaylistForm.NextTrack;
  end;
  if SoundModified then begin
    if Assigned(TheStream) then FSOUND_Stream_Close(TheStream);
    with CDevice do SoundDevice:=Items[ItemIndex];
    try SoundRate:=StrToInt(CRate.Text); except SoundRate:=0; end;
    if (SoundRate<8000) OR (SoundRate>65536) then SoundRate:=44100;
    case COutput.ItemIndex of
      1: SoundOutput:=Ord(FSOUND_OUTPUT_NOSOUND);
      2: SoundOutput:=Ord(FSOUND_OUTPUT_WINMM);
      3: SoundOutput:=Ord(FSOUND_OUTPUT_DSOUND);
      4: SoundOutput:=Ord(FSOUND_OUTPUT_A3D);
      5: SoundOutput:=Ord(FSOUND_OUTPUT_ASIO);
    else
      SoundOutput:=-1;
    end;
  end;

  SaveSettings;

  if SoundModified then begin
    FSOUND_Close();
    PlaylistForm.InitSound;
    if Assigned(PlayingNow) then
      PlaylistForm.PlayItem(PlayingNow^.MyIndex);
  end;
end;

procedure TSettingsForm.DIncludeClick(Sender: TObject);
begin
  BDelInclude.Enabled:=(DInclude.ItemIndex>=0);
end;

procedure TSettingsForm.DExcludeClick(Sender: TObject);
begin
  BDelExclude.Enabled:=(DExclude.ItemIndex>=0);
end;

procedure TSettingsForm.BAddIncludeClick(Sender: TObject);
begin
  if not Assigned(DirSelectForm) then
    Application.CreateForm(TDirSelectForm,DirSelectForm);
  if DirSelectForm.ShowModal<>mrOK then exit;
  DInclude.Items.Add(ExcludeTrailingPathDelimiter(DirSelectForm.Directory));
  ListModified:=1;
end;

procedure TSettingsForm.BAddExcludeClick(Sender: TObject);
begin
  if not Assigned(DirSelectForm) then
    Application.CreateForm(TDirSelectForm,DirSelectForm);
  if DirSelectForm.ShowModal<>mrOK then exit;
  DExclude.Items.Add(ExcludeTrailingPathDelimiter(DirSelectForm.Directory));
  ListModified:=1;
end;

procedure TSettingsForm.BDelIncludeClick(Sender: TObject);
var i:integer;
begin
  i:=DInclude.ItemIndex;
  if i<0 then exit;
  DInclude.Items.Delete(i);
  ListModified:=1;
end;

procedure TSettingsForm.BDelExcludeClick(Sender: TObject);
var i:integer;
begin
  i:=DExclude.ItemIndex;
  if i<0 then exit;
  DExclude.Items.Delete(i);
  ListModified:=1;
end;

procedure TSettingsForm.SoundChange(Sender: TObject);
begin
  SoundModified:=true;
end;

end.
