unit Update;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Globals, ExtCtrls, StdCtrls;

const PieDetail=100;

type
  TUpdateForm = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    StatusPie: TPaintBox;
    procedure FormCreate(Sender: TObject);
    procedure StatusPiePaint(Sender: TObject);
  private
    { Private declarations }
    PiePos:integer;
    PieCol1,PieCol2:TColor;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  public
    { Public declarations }
    procedure Step;
  end;

var
  UpdateForm: TUpdateForm;

procedure UpdatePlaylist(FullUpdate:boolean);
procedure SetSortOrder(NewOrder:TSortOrder);

implementation
uses PLMan, Playlist, GetInfo;

{$R *.dfm}

procedure TUpdateForm.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style:=Params.Style OR WS_DLGFRAME;
end;

procedure TUpdateForm.FormCreate(Sender: TObject);
begin
  PiePos:=0;
  PieCol1:=$FFC000;
  PieCol2:=$806000;
end;

procedure TUpdateForm.StatusPiePaint(Sender: TObject);
var X,Y:integer;
begin with StatusPie.Canvas do begin
  X:=20+round(24.0*sin(PiePos/PieDetail*2.0*Pi));
  Y:=20-round(24.0*cos(PiePos/PieDetail*2.0*Pi));
  Pen.Style:=psClear;
  Brush.Color:=PieCol1;
  Pie(0,0,40,40,X,Y,20,0);
  Brush.Color:=PieCol2;
  Pie(0,0,40,40,20,0,X,Y);
end; end;

procedure TUpdateForm.Step;
var Temp:TColor;
begin
  inc(PiePos);
  if PiePos>=PieDetail then begin
    dec(PiePos,PieDetail);
    Temp:=PieCol1; PieCol1:=PieCol2; PieCol2:=Temp;
  end;
  StatusPiePaint(nil);
end;

////////////////////////////////////////////////////////////////////////////////

var OldPL:TPlaylist;
    Full:boolean;

procedure AddDirectory(const Dir:string);
var Cont:integer; SR:TSearchRec; Item:TPlaylistItem;
    Ext:string; Index:integer;
begin
  if ExcludeDirs.IndexOf(Dir)>=0 then exit;
  Cont:=FindFirst(Dir+'\*.*',faAnyFile,SR);
  while Cont=0 do begin
//writeln(Dir,'\',SR.Name);
    if (SR.Attr AND faDirectory)<>0 then begin
      if (SR.Name<>'.') AND (SR.Name<>'..') then
        AddDirectory(Dir+'\'+SR.Name);
    end else begin
      Ext:=LowerCase(ExtractFileExt(SR.Name));
      if (Ext='.mp3') OR (Ext='.ogg') OR (Ext='.wma') OR (Ext='.wav') OR (Ext='.mp2') then begin
        UpdateForm.Step;
        ClearPlaylistItem(Item);
        Item.FileName:=Dir+'\'+SR.Name;
        Item.FileDate:=SR.Time;
        Item.FileSize:=SR.Size;
        Index:=OldPL.Find(Item);
        if Index>=0 then begin
          Item:=OldPL[Index]^;
          GetSongInfo(Item,Ext,nil,gsiFast);
        end else
          GetSongInfo(Item,Ext,nil,gsiFast);
        MainPL.Add(Item);
      end;
    end;
    Cont:=FindNext(SR);
  end;
//writeln('*** Directory ',Dir,' is done.');
  FindClose(SR);
end;

procedure UpdatePlaylist(FullUpdate:boolean);
var i:integer;
begin
  UpdateForm:=TUpdateForm.Create(nil);
  UpdateForm.Show;
  UpdateForm.Repaint;
  Full:=FullUpdate;
  if Assigned(MainPL) then begin
    OldPL:=TPlaylist.CreateFromPlaylist(MainPL);
    MainPL.Free;
  end else
    OldPL:=TPlaylist.Create;
  MainPL:=TPlaylist.Create;
  for i:=0 to IncludeDirs.Count-1 do
    AddDirectory(IncludeDirs[i]);
  OldPL.Free;
  PlaylistForm.LList.Items.Clear;
  for i:=1 to MainPL.Count do
    PlaylistForm.LList.Items.Add('');
  UpdateForm.Step;
  SetSortOrder(SortOrder);
  FreeAndNil(UpdateForm);
end;

procedure SortByTitle(max:integer);
var a,b,t:integer;
begin
  for a:=0 to max-1 do
    for b:=a+1 to max do
      if CompareText(MainPL[Order[a]]^.Title,MainPL[Order[b]]^.Title)>0 then begin
        t:=Order[a]; Order[a]:=Order[b]; Order[b]:=t;
      end;
end;

function CompareByRating(const a,b:TPlaylistItem):integer;
var d:real;
begin
  d:=RateItem(a)-RateItem(b);
  if d<0.0 then begin Result:=+1; exit; end;
  if d>0.0 then begin Result:=-1; exit; end;
  Result:=b.PlayCount-a.PlayCount;
  if Result=0 then Result:=CompareText(a.Title,b.Title);
end;

procedure SortByRating(max:integer);
var a,b,t:integer;
begin
  for a:=0 to max-1 do
    for b:=a+1 to max do
      if CompareByRating(MainPL[Order[a]]^,MainPL[Order[b]]^)>0 then begin
        t:=Order[a]; Order[a]:=Order[b]; Order[b]:=t;
      end;
end;

procedure SetSortOrder(NewOrder:TSortOrder);
var max,i:integer;
begin
  SortOrder:=NewOrder;
  max:=MainPL.Count-1;
  SetLength(Order,max+1);
  for i:=0 to max do Order[i]:=i;
  if SortOrder=soTitle then SortByTitle(max)
                       else SortByRating(max);
end;

end.
